@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -108,14 +108,10 @@ void
 ShmWriter::vappendf(const char *fmt, va_list ap)
 {
     SBuf vaBuf;
-#if defined(VA_COPY)
     va_list apCopy;
-    VA_COPY(apCopy, ap);
+    va_copy(apCopy, ap);
     vaBuf.vappendf(fmt, apCopy);
     va_end(apCopy);
-#else
-    vaBuf.vappendf(fmt, ap);
-#endif
     append(vaBuf.rawContent(), vaBuf.length());
 }
 
@@ -321,21 +317,20 @@ MemStore::get(const cache_key *key)
     StoreEntry *e = new StoreEntry();
 
     // XXX: We do not know the URLs yet, only the key, but we need to parse and
-    // store the response for the Root().get() callers to be happy because they
+    // store the response for the Root().find() callers to be happy because they
     // expect IN_MEMORY entries to already have the response headers and body.
-    e->makeMemObject();
+    e->createMemObject();
 
     anchorEntry(*e, index, *slot);
 
     const bool copied = copyFromShm(*e, index, *slot);
 
-    if (copied) {
-        e->hashInsert(key);
+    if (copied)
         return e;
-    }
 
-    debugs(20, 3, HERE << "mem-loading failed; freeing " << index);
+    debugs(20, 3, "failed for " << *e);
     map->freeEntry(index); // do not let others into the same trap
+    destroyStoreEntry(static_cast<hash_link *>(e));
     return NULL;
 }
 
@@ -379,7 +374,7 @@ MemStore::updateHeadersOrThrow(Ipc::StoreMapUpdate &update)
 
     Must(update.stale.anchor);
     ShmWriter writer(*this, update.entry, update.fresh.fileNo);
-    reply.packHeadersInto(&writer);
+    reply.packHeadersUsingSlowPacker(writer);
     const uint64_t freshHdrSz = writer.totalWritten;
     debugs(20, 7, "fresh hdr_sz: " << freshHdrSz << " diff: " << (freshHdrSz - staleHdrSz));
 
@@ -403,46 +398,41 @@ MemStore::updateHeadersOrThrow(Ipc::StoreMapUpdate &update)
 }
 
 bool
-MemStore::anchorCollapsed(StoreEntry &collapsed, bool &inSync)
+MemStore::anchorToCache(StoreEntry &entry, bool &inSync)
 {
     if (!map)
         return false;
 
     sfileno index;
     const Ipc::StoreMapAnchor *const slot = map->openForReading(
-            reinterpret_cast<cache_key*>(collapsed.key), index);
+            reinterpret_cast<cache_key*>(entry.key), index);
     if (!slot)
         return false;
 
-    anchorEntry(collapsed, index, *slot);
-    inSync = updateCollapsedWith(collapsed, index, *slot);
+    anchorEntry(entry, index, *slot);
+    inSync = updateAnchoredWith(entry, index, *slot);
     return true; // even if inSync is false
 }
 
 bool
-MemStore::updateCollapsed(StoreEntry &collapsed)
+MemStore::updateAnchored(StoreEntry &entry)
 {
-    assert(collapsed.mem_obj);
-
-    const sfileno index = collapsed.mem_obj->memCache.index;
-
-    // already disconnected from the cache, no need to update
-    if (index < 0)
-        return true;
-
     if (!map)
         return false;
 
+    assert(entry.mem_obj);
+    assert(entry.hasMemStore());
+    const sfileno index = entry.mem_obj->memCache.index;
     const Ipc::StoreMapAnchor &anchor = map->readableEntry(index);
-    return updateCollapsedWith(collapsed, index, anchor);
+    return updateAnchoredWith(entry, index, anchor);
 }
 
-/// updates collapsed entry after its anchor has been located
+/// updates Transients entry after its anchor has been located
 bool
-MemStore::updateCollapsedWith(StoreEntry &collapsed, const sfileno index, const Ipc::StoreMapAnchor &anchor)
+MemStore::updateAnchoredWith(StoreEntry &entry, const sfileno index, const Ipc::StoreMapAnchor &anchor)
 {
-    collapsed.swap_file_sz = anchor.basics.swap_file_sz;
-    const bool copied = copyFromShm(collapsed, index, anchor);
+    entry.swap_file_sz = anchor.basics.swap_file_sz;
+    const bool copied = copyFromShm(entry, index, anchor);
     return copied;
 }
 
@@ -450,15 +440,8 @@ MemStore::updateCollapsedWith(StoreEntry &collapsed, const sfileno index, const
 void
 MemStore::anchorEntry(StoreEntry &e, const sfileno index, const Ipc::StoreMapAnchor &anchor)
 {
-    const Ipc::StoreMapAnchor::Basics &basics = anchor.basics;
-
-    e.swap_file_sz = basics.swap_file_sz;
-    e.lastref = basics.lastref;
-    e.timestamp = basics.timestamp;
-    e.expires = basics.expires;
-    e.lastmod = basics.lastmod;
-    e.refcount = basics.refcount;
-    e.flags = basics.flags;
+    assert(!e.hasDisk()); // no conflict with disk entry basics
+    anchor.exportInto(e);
 
     assert(e.mem_obj);
     if (anchor.complete()) {
@@ -470,11 +453,7 @@ MemStore::anchorEntry(StoreEntry &e, const sfileno index, const Ipc::StoreMapAnc
         assert(e.mem_obj->object_sz < 0);
         e.setMemStatus(NOT_IN_MEMORY);
     }
-    assert(e.swap_status == SWAPOUT_NONE); // set in StoreEntry constructor
-    e.ping_status = PING_NONE;
 
-    EBIT_CLR(e.flags, RELEASE_REQUEST);
-    EBIT_CLR(e.flags, KEY_PRIVATE);
     EBIT_SET(e.flags, ENTRY_VALIDATED);
 
     MemObject::MemCache &mc = e.mem_obj->memCache;
@@ -577,7 +556,6 @@ MemStore::copyFromShmSlice(StoreEntry &e, const StoreIOBuffer &buf, bool eof)
         const int result = rep->httpMsgParseStep(mb.buf, buf.length, eof);
         if (result > 0) {
             assert(rep->pstate == psParsed);
-            EBIT_CLR(e.flags, ENTRY_FWD_HDR_WAIT);
         } else if (result < 0) {
             debugs(20, DBG_IMPORTANT, "Corrupted mem-cached headers: " << e);
             return false;
@@ -625,16 +603,8 @@ MemStore::shouldCache(StoreEntry &e) const
     }
 
     const int64_t expectedSize = e.mem_obj->expectedReplySize(); // may be < 0
-
-    // objects of unknown size are not allowed into memory cache, for now
-    if (expectedSize < 0) {
-        debugs(20, 5, "Unknown expected size: " << e);
-        return false;
-    }
-
     const int64_t loadedSize = e.mem_obj->endOffset();
     const int64_t ramSize = max(loadedSize, expectedSize);
-
     if (ramSize > maxObjectSize()) {
         debugs(20, 5, HERE << "Too big max(" <<
                loadedSize << ", " << expectedSize << "): " << e);
@@ -674,7 +644,10 @@ MemStore::startCaching(StoreEntry &e)
     e.mem_obj->memCache.index = index;
     e.mem_obj->memCache.io = MemObject::ioWriting;
     slot->set(e);
-    map->startAppending(index);
+    // Do not allow others to feed off an unknown-size entry because we will
+    // stop swapping it out if it grows too large.
+    if (e.mem_obj->expectedReplySize() >= 0)
+        map->startAppending(index);
     e.memOutDecision(true);
     return true;
 }
@@ -683,15 +656,9 @@ MemStore::startCaching(StoreEntry &e)
 void
 MemStore::copyToShm(StoreEntry &e)
 {
-    // prevents remote readers from getting ENTRY_FWD_HDR_WAIT entries and
-    // not knowing when the wait is over
-    if (EBIT_TEST(e.flags, ENTRY_FWD_HDR_WAIT)) {
-        debugs(20, 5, "postponing copying " << e << " for ENTRY_FWD_HDR_WAIT");
-        return;
-    }
-
     assert(map);
     assert(e.mem_obj);
+    Must(!EBIT_TEST(e.flags, ENTRY_FWD_HDR_WAIT));
 
     const int64_t eSize = e.mem_obj->endOffset();
     if (e.mem_obj->memCache.offset >= eSize) {
@@ -700,6 +667,9 @@ MemStore::copyToShm(StoreEntry &e)
         return; // nothing to do (yet)
     }
 
+    // throw if an accepted unknown-size entry grew too big or max-size changed
+    Must(eSize <= maxObjectSize());
+
     const int32_t index = e.mem_obj->memCache.index;
     assert(index >= 0);
     Ipc::StoreMapAnchor &anchor = map->writeableEntry(index);
@@ -777,6 +747,7 @@ MemStore::nextAppendableSlice(const sfileno fileNo, sfileno &sliceOffset)
             slice.next = sliceOffset = reserveSapForWriting(page);
             extras->items[sliceOffset].page = page;
             debugs(20, 7, "entry " << fileNo << " new slice: " << sliceOffset);
+            continue; // to get and return the slice at the new sliceOffset
         }
 
         return slice;
@@ -904,32 +875,36 @@ MemStore::completeWriting(StoreEntry &e)
 
     e.mem_obj->memCache.index = -1;
     e.mem_obj->memCache.io = MemObject::ioDone;
-    map->closeForWriting(index, false);
+    map->closeForWriting(index);
 
     CollapsedForwarding::Broadcast(e); // before we close our transient entry!
     Store::Root().transientsCompleteWriting(e);
 }
 
 void
-MemStore::markForUnlink(StoreEntry &e)
+MemStore::evictCached(StoreEntry &e)
 {
-    assert(e.mem_obj);
-    if (e.mem_obj->memCache.index >= 0)
-        map->freeEntry(e.mem_obj->memCache.index);
+    debugs(47, 5, e);
+    if (e.hasMemStore()) {
+        if (map->freeEntry(e.mem_obj->memCache.index))
+            CollapsedForwarding::Broadcast(e);
+        if (!e.locked()) {
+            disconnect(e);
+            e.destroyMemObject();
+        }
+    } else if (const auto key = e.publicKey()) {
+        // the entry may have been loaded and then disconnected from the cache
+        evictIfFound(key);
+        if (!e.locked())
+            e.destroyMemObject();
+    }
 }
 
 void
-MemStore::unlink(StoreEntry &e)
+MemStore::evictIfFound(const cache_key *key)
 {
-    if (e.mem_obj && e.mem_obj->memCache.index >= 0) {
-        map->freeEntry(e.mem_obj->memCache.index);
-        disconnect(e);
-    } else if (map) {
-        // the entry may have been loaded and then disconnected from the cache
-        map->freeEntryByKey(reinterpret_cast<cache_key*>(e.key));
-    }
-
-    e.destroyMemObject(); // XXX: but it may contain useful info such as a client list. The old code used to do that though, right?
+    if (map)
+        map->freeEntryByKey(key);
 }
 
 void
@@ -937,12 +912,12 @@ MemStore::disconnect(StoreEntry &e)
 {
     assert(e.mem_obj);
     MemObject &mem_obj = *e.mem_obj;
-    if (mem_obj.memCache.index >= 0) {
+    if (e.hasMemStore()) {
         if (mem_obj.memCache.io == MemObject::ioWriting) {
             map->abortWriting(mem_obj.memCache.index);
             mem_obj.memCache.index = -1;
             mem_obj.memCache.io = MemObject::ioDone;
-            Store::Root().transientsAbandon(e); // broadcasts after the change
+            Store::Root().stopSharing(e); // broadcasts after the change
         } else {
             assert(mem_obj.memCache.io == MemObject::ioReading);
             map->closeForReading(mem_obj.memCache.index);
